<?php

    class Previous_Action_Results_Condition extends Reactor_Condition
    {
        protected $tpl = 'previous_action_results.php';
        protected $name = 'Previous event chain action result(s) is/are';
		
		public static $registration = array(
			CONDITION_NAME => 'Previous action results',
			CONDITION_CATEGORY => 'Nagios Reactor',
			CONDITION_DESCRIPTION => 'Looks at results of actions run previously in the chain.',
			CONDITION_AUTHOR => 'Jeremy Yunis, Nagios Enterprises, LLC',
			CONDITION_VERSION => '1.0',
			CONDITION_DATE => '08/28/2013', //Last version update
			CONDITION_AUTHLEVEL => 0,  // 0 - everyone | 1 - Admin only
			CONDITION_TYPE => 'core', //core | user. core modules can't be deleted
			CONDITION_META => array(),	//Misc information	
		);
		
        protected $fields = array(
            'threshold' => array(
                'label' => 'threshold',
                'validate' => 'trim|required|numeric|callback_check_threshold',
            ),
            'history_count' => array(
                'label' => 'count',
                'validate' => 'trim|required|numeric',
            ),
            'comparison_type' => array(
                'label' => 'result',
                'validate' => 'trim|required'
            ),
            'custom_value' => array(
                'label' => 'exit code',
                'validate' => 'trim|numeric|callback_check_custom_value'
            ),
        );

        public function check_threshold($str)
        {
            $threshold = $this->ci->input->post('threshold');
            $history_count = $this->ci->input->post('history_count');
            if($threshold > $history_count) {
                $this->ci->form_validation->set_message('check_threshold', _('The first value should be smaller than the second.'));
                return FALSE;
            }
            return TRUE;
        }

        public function check_custom_value($str)
        {
            $type = $this->ci->input->post('comparison_type');
            if($type == 'custom_value') {
                if(!strlen($str)) {
                    $this->ci->form_validation->set_message('check_custom_value', _('This field is required.'));
                    return FALSE;
                }
            }
            return TRUE;
        }

        public function get_label()
        {
            if($params = $this->get_params()) {
                if($params['comparison_type'] == 'ok') {
                    return sprintf('%d of the last %d actions finished successfully', $params['threshold'], $params['history_count']);
                } elseif($params['comparison_type'] == 'err') {
                    return sprintf('%d of the last %d actions finished with errors', $params['threshold'], $params['history_count']);
                } elseif($params['comparison_type'] == 'custom_value') {
                    return sprintf('%d of the last %d actions finished with code %d', $params['threshold'], $params['history_count'], $params['custom_value']);
                }
            }
            return parent::get_label();
        }
       
        public function run($context)
        {
            $this->set_context($context);
            $params = $this->get_params();

            $prev_results = $this->get_context_var('__actionresult__');
            if(count($prev_results) < $params['history_count']) {
                $this->log('info', _('There are not enough past action results to perform this check.'));
                $this->set_result(FALSE);
                return;
            }

            $result_set = array_slice($prev_results, $params['history_count'] * -1);
            
            switch($params['comparison_type']) {
                case "ok":
                    $fn = function($val) {
                        return $val === 0 || $val === "0";
                    };
                    break;
                case "err":
                    $fn = function($val) {
                        return $val > 0;
                    };
                    break;
                case "custom_value":
                    $fn = function($val) {
                        return $val == $params['custom_value'];
                    };
                    break;
                default:
                    $fn = function($val) { return false; };
                    break;
            }

            $result = count(array_filter($result_set, $fn)) >= $params['threshold'];

            $this->set_result((bool) $result);

        }

    }